//Road Rage Middle Finger Machine (source code)
//By GRoVer Blom
//https://www.instructables.com/member/groovyuploader/
//https://game-junk.itch.io
//https://www.youtube.com/channel/UCu2N_sTUE72OnngrReycVkA


//Libraries
#include <Servo.h>

//State variables
int Machine_state = 0; //0 = closed, 1 = open
int Button_state = 0; //0 = off, 1 = on

//Pin distrubution varibles
int Button_pin = 3;
int Servo_finger_A_pin = 4;
int Servo_finger_B_pin = 5;
int Servo_wing_A_pin = 6;
int Servo_wing_B_pin = 7;

//Servo variables
Servo Servo_finger_A;
Servo Servo_finger_B;
Servo Servo_wing_A;
Servo Servo_wing_B;

//Additional variables
int Hand_servo_delay = 250; 
int Current_distribute_delay = 20;
int A_open_rot_value = 180;
int B_open_rot_value = 0;


void setup() 
{
  set_all_pin_modi();
  attach_all_servo();
  //set_all_servos_to_zer0();
  //open_wings();
  //close_wings();
  //draw_middle_finger();
  //withdraw_middle_finger();
}

void loop() 
{
  check_button_state(); //check if button is pressed
}



//Main functions
void check_button_state()
{
Button_state = digitalRead(Button_pin); //Update if button is pressed
  if (Machine_state == 0 && Button_state == HIGH) //Check if button is pressed (Button_state) & if machine is already open/close (Machine_state, 0 = closed & 1 = open), if its closed it will open
    {
      open_wings();
      delay(Hand_servo_delay); //wait for wings to open
      draw_middle_finger();
      Machine_state = 1; //keep track of machine is open or closed (open)
    }
  else
    {
      if (Machine_state == 1 && Button_state == LOW) //Check if button is pressed (Button_state) & if machine is already open/close (Machine_state, 0 = closed & 1 = open), if its open it will close
        {
          withdraw_middle_finger();
          delay(Hand_servo_delay); //Wait for hand to withdraw
          close_wings();
          Machine_state = 0; //Keep track of machine is open or closed (closed)
        }
    }
}


//Additional functions
void set_all_pin_modi()
{
  pinMode(Button_pin, INPUT);
  pinMode(Servo_finger_A_pin, OUTPUT);
  pinMode(Servo_finger_B_pin, OUTPUT);
  pinMode(Servo_wing_A_pin, OUTPUT);
  pinMode(Servo_wing_B_pin, OUTPUT);
}


void attach_all_servo()
{
  Servo_finger_A.attach(Servo_finger_A_pin);
  Servo_finger_B.attach(Servo_finger_B_pin);
  Servo_wing_A.attach(Servo_wing_A_pin);
  Servo_wing_B.attach(Servo_wing_B_pin);
}



//Test/Calibrate/Maintance functions
void set_all_servos_to_zer0() //Set all servo's to zer0 before assembly
{
  Servo_finger_A.write(90);
  Servo_finger_B.write(90);
  Servo_wing_A.write(90); 
  Servo_wing_B.write(90);
}

void open_wings() //Open wings manual (for maintance purpuses)
{
  Servo_wing_A.write(A_open_rot_value); 
  Servo_wing_B.write(B_open_rot_value);
}

void close_wings() //close wings manual (for maintance purpuses)
{
  Servo_wing_A.write(90); 
  Servo_wing_B.write(90);
}

void draw_middle_finger() //Draw finger manual (for maintance purpuses)
{
  Servo_finger_A.write(A_open_rot_value);
  Servo_finger_B.write(B_open_rot_value);
}

void withdraw_middle_finger() //Withdraw finger manual (for maintance purpuses)
{
  Servo_finger_A.write(90);
  Servo_finger_B.write(90);
}